#include "mti_lzss.hpp"

typedef typename mti_lzss::size_type size_type;

//// FIND PATTERN /////////////////////////////////////////////////////////////

typedef mti_lzss::occ_list occ_list;

occ_list mti_lzss::findPattern(const unsigned char *p, size_type m) {
	//let other self index do the job
	auto occs = csi.findPattern(p, m);
	//and associate each occurence to the right pattern
	occ_list occurences;

	for (auto occ : occs) {
		//find text to which occurence is associated
		auto it = std::upper_bound(offsets.begin(), offsets.end(), occ);
		size_type text = it - offsets.begin() - 1;
		size_type textoffset = occ - offsets[text];
		//be sure occurence does not touch a border
		if (occ + m < offsets[text+1]) {
			occurences.push_back( std::pair<size_type,size_type>(text, textoffset) );
		}
	}
	return occurences;
}

//// SERIALIZATION AND LOAD ///////////////////////////////////////////////////

size_type mti_lzss::serialize(std::ostream& out, sdsl::structure_tree_node* v,
		std::string name) const {
	sdsl::structure_tree_node *child = sdsl::structure_tree::add_child(
		                                     v, name, sdsl::util::class_name(*this));
	size_type written_bytes = 0;
	written_bytes += csi.serialize(out, child, "csi");

	//serialize offset vector
	sdsl::structure_tree_node *o_child = sdsl::structure_tree::add_child(
		                                   child, "offsets", sdsl::util::class_name(offsets));
	size_type off_size = offsets.size();
	out.write((char *)&off_size, sizeof(size_type));
	out.write((char *)offsets.data(), sizeof(size_type) * offsets.size());
	size_type o_bytes = (offsets.size() + 1) * sizeof(size_type);
	sdsl::structure_tree::add_size(o_child, o_bytes);

	written_bytes += o_bytes;

	sdsl::structure_tree::add_size(child, written_bytes);
	return written_bytes;
}

void mti_lzss::load(std::istream& in) {
	csi.load(in);

	//load offset vector
	size_type off_size;
	in.read((char *)&off_size, sizeof(size_type));
	offsets.resize( off_size );
	in.read((char *)offsets.data(), sizeof(size_type) * offsets.size());
}
