#include <algorithm>
#include <istream>
#include <iterator>
#include <iostream>
#include <list>
#include <limits>
#include <memory>
#include <sstream>
#include <utility>
#include "../index/mti_lzss.hpp"

using namespace std;

int main() {
	typedef typename mti_lzss::size_type size_type;

	//input strings to build index for
	list<unique_ptr<istream>> stringstreams;
	cout << "Enter strings to construct index from (empty string indicates end of input): " << endl;
	string str = "";
	cout << "(1): ";
	size_type i = 2;
	for (getline(cin, str); !str.empty(); getline(cin, str)) {
		cout << "(" << i++ << "): ";
		stringstreams.push_back( unique_ptr<istream>( new istringstream(str) ) );
	}
	
	//construct selfindex from strings
	mti_lzss index(stringstreams.begin(), stringstreams.end());
	cout << "Size of index in bytes: " << sdsl::size_in_bytes(index) << endl;
	//let user search for patterns until he don't want no more
	cout << "Pattern to search for (empty pattern quits): ";
	string pattern = "";
	for (getline(cin, pattern); !pattern.empty(); pattern = "", getline(cin, pattern)) {
		//Find pattern in strings
		auto l_occ = index.findPattern(pattern);

		//sort them by text and index
		vector<std::pair<size_type,size_type>> v_occ(l_occ.begin(), l_occ.end());
		sort(v_occ.begin(), v_occ.end());

		//print occurence count
		cout << "\t" << v_occ.size() << " occurences";

		//print texts where pattern occurs and pattern positions
		auto text_it = stringstreams.begin();
		size_type lasttext = 0;
		size_type lastpos  = std::numeric_limits<size_type>::max();
		for (auto &occ : v_occ) {
			size_type text     = occ.first;
			size_type textpos  = occ.second;
			if (text != lasttext || //output new text
			    lastpos == std::numeric_limits<size_type>::max()) { //for base case
				cout << endl << "\tAt Text " << text+1 << ":" << endl;
				//advance iterator to text's element
				std::advance( text_it, text - lasttext );
				//and print text
				cout << "\t" << ((istringstream *)text_it->get())->str() << endl;
				cout << "\t";
				lasttext = text;
				lastpos = 0;
			}
			//output occurence underlined
			while (lastpos < textpos) {
				cout << ' ';
				++lastpos;
			}
			cout << '^';
			++lastpos;
		}
		cout << endl << "Pattern to search for (empty pattern quits): ";
	}
	return 0;
}
