#include <ios>
#include <iostream>
#include <istream>
#include <list>
#include <memory>
#include <sstream>
#include <string>
#include "../index/mti_lzss.hpp"

using namespace std;

int main() {
	typedef typename mti_lzss::size_type size_type;

	//input strings to build index for
	list<unique_ptr<istream>> stringstreams;
	cout << "Enter strings to construct index from (empty string indicates end of input): " << endl;
	string str = "";
	cout << "(1): ";
	size_type i = 2;
	for (getline(cin, str); !str.empty(); getline(cin, str)) {
		cout << "(" << i++ << "): ";
		stringstreams.push_back( unique_ptr<istream>( new istringstream(str) ) );
	}
	
	//construct selfindex from strings
	mti_lzss index1(stringstreams.begin(), stringstreams.end());

	//serialize to a stringstream
	stringstream ss;
	index1.serialize(ss);

	//create a second index by loading from stream
	ss.seekg(0, ios_base::beg);
	mti_lzss index2;
	index2.load(ss);

	//output sizes of both indexes
	cout << "Size of first index in bytes:  " << sdsl::size_in_bytes(index1) << endl;
	cout << "Size of second index in bytes: " << sdsl::size_in_bytes(index2) << endl;

	//test if both strings are equal.
	//test is simple: Compare occurences from index and index2 with all
	// strings that were read at start of input
	for (auto &ptr : stringstreams) {
		str = ((istringstream *)ptr.get())->str();
		auto list1 = index1.findPattern(str);
		auto list2 = index2.findPattern(str);
		if (list1 != list2) {
			cout << "Serialization failed" << endl;
			return 1;
		}
	}
	cout << "Seralization worked" << endl;
	return 0;
}
